#!/usr/bin/env python

"""
Filesystem utilities.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import errno
from imiptools.config import DEFAULT_PERMISSIONS, DEFAULT_DIR_PERMISSIONS
from os.path import abspath, commonprefix, exists, join
from os import chmod, makedirs, mkdir, rmdir
from time import sleep, time

def check_dir(base, dir):
    return commonprefix([base, abspath(dir)]) == base

def fix_permissions(filename, is_dir=False):
    try:
        chmod(filename, is_dir and DEFAULT_DIR_PERMISSIONS or DEFAULT_PERMISSIONS)
    except OSError:
        pass

class FileBase:

    "Basic filesystem operations."

    lock_name = "__lock__"

    def __init__(self, store_dir):
        self.store_dir = store_dir
        if not exists(self.store_dir):
            makedirs(self.store_dir, DEFAULT_DIR_PERMISSIONS)

    def get_file_object(self, base, *parts):
        pathname = join(base, *parts)
        return check_dir(base, pathname) and pathname or None

    def get_object_in_store(self, *parts):

        """
        Return the name of any valid object stored within a hierarchy specified
        by the given 'parts'.
        """

        parent = expected = self.store_dir

        for part in parts:
            filename = self.get_file_object(expected, part)
            if not filename:
                return False
            parent = expected
            expected = filename

        if not exists(parent):
            makedirs(parent, DEFAULT_DIR_PERMISSIONS)

        return filename

    # Locking methods.
    # This uses the directory creation method exploited by MoinMoin.util.lock.
    # However, a simple single lock type mechanism is employed here.

    def get_lock_dir(self, *parts):
        parts = parts and list(parts) or []
        parts.append(self.lock_name)
        return self.get_object_in_store(*parts)

    def acquire_lock(self, timeout=None, *parts):

        """
        Acquire an exclusive lock on the directory or a path within it described
        by 'parts'.
        """

        start = now = time()

        while not timeout or now - start < timeout:
            try:
                mkdir(self.get_lock_dir(*parts))
                break
            except OSError, exc:
                if exc.errno != errno.EEXIST:
                    raise
            sleep(1)
            now = time()

    def release_lock(self, *parts):

        """
        Release an acquired lock on the directory or a path within it described
        by 'parts'.
        """

        try:
            rmdir(self.get_lock_dir(*parts))
        except OSError, exc:
            if exc.errno != errno.ENOENT:
                raise

# vim: tabstop=4 expandtab shiftwidth=4
